@@ -107,7 +107,7 @@ except ImportError:
                           " install ordereddict 1.1 from pypi for python2.6")
 
 __pkgname__ = 'python-crontab'
-__version__ = '2.3.9'
+__version__ = '2.4.1'
 
 ITEMREX = re.compile(r'^\s*([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)'
                      r'\s+([^@#\s]+)\s+([^\n]*?)(\s+#\s*([^\n]*)|$)')
@@ -226,6 +226,12 @@ class CronTab(object):
         self.intab = tab
         self.read(tabfile)
         self._log = log
+    
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.write()
 
     @property
     def log(self):
@@ -343,17 +349,19 @@ class CronTab(object):
 
         if not self.filen:
             # Add the entire crontab back to the user crontab
-            if self.user:
-                proc = open_pipe(CRONCMD, path, **self.user_opt)
-                # This could do with being cleaned up quite a bit
-                proc.wait()
-                proc.stdout.close()
-                proc.stderr.close()
-                os.unlink(path)
-            else:
+            if not self.user:
                 os.unlink(path)
                 raise IOError("Please specify user or filename to write.")
 
+            proc = open_pipe(CRONCMD, path, **self.user_opt)
+            ret = proc.wait()
+            if ret != 0:
+                raise IOError("Program Error: {} returned {}: {}".format(
+                    CRONCMD, ret, proc.stderr.read()))
+            proc.stdout.close()
+            proc.stderr.close()
+            os.unlink(path)
+
     def write_to_user(self, user=True):
         """Write the crontab to a user (or root) instead of a file."""
         return self.write(user=user)
@@ -404,7 +412,7 @@ class CronTab(object):
             result += u'\n'
         return result
 
-    def new(self, command='', comment='', user=None):
+    def new(self, command='', comment='', user=None, pre_comment=False):
         """
         Create a new cron with a command and comment.
 
@@ -412,7 +420,7 @@ class CronTab(object):
         """
         if not user and self.user is False:
             raise ValueError("User is required for system crontabs.")
-        item = CronItem(command, comment, user=user, cron=self)
+        item = CronItem(command, comment, user=user, cron=self, pre_comment=pre_comment)
         self.append(item)
         return item
 
@@ -542,7 +550,7 @@ class CronItem(object):
     An item which objectifies a single line of a crontab and
     May be considered to be a cron job object.
     """
-    def __init__(self, command='', comment='', user=None, cron=None):
+    def __init__(self, command='', comment='', user=None, cron=None, pre_comment=False):
         self.cron = cron
         self.user = user
         self.valid = False
@@ -555,7 +563,7 @@ class CronItem(object):
 
         # Marker labels Ansible jobs etc
         self.marker = None
-        self.pre_comment = False
+        self.pre_comment = pre_comment
         self._log = None
 
         # Initalise five cron slices using static info.
@@ -567,6 +575,14 @@ class CronItem(object):
             self.set_command(command)
             self.valid = True
 
+    def __hash__(self):
+        return hash((self.command, self.comment, self.hour, self.minute, self.dow))
+
+    def __eq__(self, other):
+        if not isinstance(other, CronItem):
+            return False
+        return self.__hash__() == other.__hash__()
+
     @classmethod
     def from_line(cls, line, user=None, cron=None):
         """Generate CronItem from a cron-line and parse out command and comment"""
@@ -1041,6 +1057,9 @@ class CronSlice(object):
         if value:
             self.parse(value)
 
+    def __hash__(self):
+        return hash(str(self))
+
     def parse(self, value):
         """Set values into the slice."""
         self.clear()
